@@ -4,27 +4,27 @@ carefulFit=function(model,traitExtension,manifestTraitvarExtension, weighting =
     #     if(transformedParams==FALSE) model$DRIFT$free[row(DRIFT$free)!=col(DRIFT$free)] <- FALSE
     
     if(traitExtension==TRUE) penalties <- OpenMx::mxAlgebra(name='penalties',
-      sum(T0VAR*T0VAR) + sum(1/(diag2vec(T0VAR) * diag2vec(T0VAR))) +
-        sum(DRIFT*DRIFT) + sum(1/(diag2vec(DRIFT) * diag2vec(DRIFT))) +
-        sum(DIFFUSION*DIFFUSION) + sum(1/(diag2vec(DIFFUSION) * diag2vec(DIFFUSION))) +
-        sum(MANIFESTVAR*MANIFESTVAR) + sum(1/(diag2vec(MANIFESTVAR) * diag2vec(MANIFESTVAR))) +
-        sum(TRAITVAR * TRAITVAR) + sum(1/(diag2vec(TRAITVAR) * diag2vec(TRAITVAR))) +
-        sum(T0TRAITEFFECT * T0TRAITEFFECT) + sum(1/(diag2vec(T0TRAITEFFECT) * diag2vec(T0TRAITEFFECT)))
+      sum(T0VAR*T0VAR) + sum(1/((diag2vec(T0VAR) * diag2vec(T0VAR)))) +
+        sum(DRIFT*DRIFT) + sum(1/(diag2vec(DRIFT) * diag2vec(DRIFT))) + sum(diag2vec(DRIFT)) +
+        sum(DIFFUSION*DIFFUSION) + sum(1/((diag2vec(DIFFUSION) * diag2vec(DIFFUSION)))) +
+        sum(MANIFESTVAR*MANIFESTVAR) + sum(1/((diag2vec(MANIFESTVAR) * diag2vec(MANIFESTVAR)))) +
+        sum(TRAITVAR * TRAITVAR) + sum(1/((diag2vec(TRAITVAR) * diag2vec(TRAITVAR)))) +
+        sum(T0TRAITEFFECT * T0TRAITEFFECT) - sum((diag2vec(T0TRAITEFFECT) * diag2vec(T0TRAITEFFECT)))
     )
 
     if(manifestTraitvarExtension==TRUE) penalties <- OpenMx::mxAlgebra(name='penalties',
-      sum(T0VAR*T0VAR) + sum(1/(diag2vec(T0VAR) * diag2vec(T0VAR))) +
-        sum(DRIFT*DRIFT) + sum(1/(diag2vec(DRIFT) * diag2vec(DRIFT))) +
-        sum(DIFFUSION*DIFFUSION) + sum(1/(diag2vec(DIFFUSION) * diag2vec(DIFFUSION))) +
-        sum(MANIFESTVAR*MANIFESTVAR) + sum(1/(diag2vec(MANIFESTVAR) * diag2vec(MANIFESTVAR))) +
-        sum(MANIFESTTRAITVAR * MANIFESTTRAITVAR) + sum(1/(diag2vec(MANIFESTTRAITVAR) * diag2vec(MANIFESTTRAITVAR)))
+      sum(T0VAR*T0VAR) + sum(1/((diag2vec(T0VAR) * diag2vec(T0VAR)))) +
+        sum(DRIFT*DRIFT) + sum(1/(diag2vec(DRIFT) * diag2vec(DRIFT))) + sum(diag2vec(DRIFT)) +
+        sum(DIFFUSION*DIFFUSION) + sum(1/((diag2vec(DIFFUSION) * diag2vec(DIFFUSION)))) +
+        sum(MANIFESTVAR*MANIFESTVAR) + sum(1/((diag2vec(MANIFESTVAR) * diag2vec(MANIFESTVAR)))) +
+        sum(MANIFESTTRAITVAR * MANIFESTTRAITVAR) + sum(1/((diag2vec(MANIFESTTRAITVAR) * diag2vec(MANIFESTTRAITVAR))))
     )
 
     if(traitExtension==FALSE & manifestTraitvarExtension==FALSE)  penalties <- OpenMx::mxAlgebra(name='penalties',
-      sum(T0VAR*T0VAR) + sum(1/(diag2vec(T0VAR) * diag2vec(T0VAR))) +
-        sum(DRIFT*DRIFT) + sum(1/(diag2vec(DRIFT) * diag2vec(DRIFT))) +
-        sum(DIFFUSION*DIFFUSION) + sum(1/(diag2vec(DIFFUSION) * diag2vec(DIFFUSION))) +
-        sum(MANIFESTVAR*MANIFESTVAR) + sum(1/(diag2vec(MANIFESTVAR) * diag2vec(MANIFESTVAR)))
+      sum(T0VAR*T0VAR) + sum(1/((diag2vec(T0VAR) * diag2vec(T0VAR)))) +
+        sum(DRIFT*DRIFT) + sum(1/(diag2vec(DRIFT) * diag2vec(DRIFT))) + sum(diag2vec(DRIFT)) +
+        sum(DIFFUSION*DIFFUSION) + sum(1/((diag2vec(DIFFUSION) * diag2vec(DIFFUSION)))) +
+        sum(MANIFESTVAR*MANIFESTVAR) + sum(1/((diag2vec(MANIFESTVAR) * diag2vec(MANIFESTVAR))))
     )
   
   # penalties <- OpenMx::mxAlgebra(name='penalties',
